@@ -3,7 +3,7 @@
   Copyright (C) 2017-2021 Dieter Baron and Thomas Klausner
 
   This file is part of libzip, a library to manipulate ZIP archives.
-  The authors can be contacted at <libzip@nih.at>
+  The authors can be contacted at <info@libzip.org>
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -74,6 +74,7 @@ zip_random_uint32(void) {
 
     if (!seeded) {
         srand((unsigned int)time(NULL));
+        seeded = true;
     }
 
     return (zip_uint32_t)rand();
